"""
use this module to initialize the whole app

Note setuptools is not picking this up
probably because this module is directly off of src
"""
import qb_core.oproc_pkg.oproc
import qb_core.rats_pkg.rats
import qb_core.ocra_pkg.ocra
from qb_core.common.plugin.peristence import PersistencePluginManager


def initialize_app():
    # since this involves importing, it's good to do this upfront
    PersistencePluginManager.initialize("cook_persistence_interface", "qb_core.rats_pkg.cook.cook_persistence_core")
    print(f'{PersistencePluginManager.get_persistence_implementation("cook_persistence_interface")}')

    qb_core.rats_pkg.rats.initialize_module()
    qb_core.oproc_pkg.oproc.initialize_module()
    qb_core.ocra_pkg.ocra.initialize_module()





import importlib


class PersistencePluginManager:
    persistence_obj_dict = {}

    @classmethod
    def initialize(cls, persistence_interface_module_name, persistence_impl_module_name):
        """
        examples
        (persistence_interface_module_name="cook_persistence_interface", module_name="qb_core.rats_pkg.cook.cook_persistence_core")
        (persistence_interface_module_name="cook_persistence_interface", module_name="cook.cook_persistence_dynamodb")
        :param persistence_interface_module_name:
        :param persistence_impl_module_name:
        :return:
        """
        module = importlib.import_module(persistence_impl_module_name)
        cls.persistence_obj_dict[persistence_interface_module_name] = module
        #
        # if persistence_interface_module_name == "core":
        #     module = importlib.import_module("qb_core.rats_pkg.cook.cook_persistence_core")
        #     cls.persistence_obj = module.get_implementation()
        # else:
        #     module = importlib.import_module("qb_core.rats_pkg.cook.cook_persistence_core")
        #     cls.persistence_obj = module.get_implementation()

    @classmethod
    def get_persistence_implementation(cls, persistence_interface_module_name):
        # return cls.persistence_obj
        module = cls.persistence_obj_dict.get(persistence_interface_module_name)
        if module is None:
            msg = f'there is no module for {persistence_interface_module_name=}. ' \
                  f'Have you defined the right name for the plugin? ' \
                  f'Or was there error in initialization of the plugin?'
            print(msg)
            raise BaseException(msg)
        else:
            return module.get_implementation()






class CookPersistenceInterface:
    def __init__(self):
        pass

    def read_cook_count(self):
        pass

    def read_cook(self, cook_name):
        pass

    def save_cook(self, cook):
        pass

    def remove_cook(self, cook):
        pass

    def print_all(self):
        pass



# from qb_core.rats_pkg.cook import _cook_persistence
#
#
# def add_cook(cook):
#     _cook_persistence.save_cook(cook)
#
#
# def remove_cook(cook):
#     _cook_persistence.remove_cook(cook)


# from qb_core.rats_pkg.cook.cook_persistence_core import CookPersistenceCore
#
# _cook_persistence = CookPersistenceCore()
from qb_core.common.plugin.peristence import PersistencePluginManager

_cook_persistence = None


def initialize():
    global _cook_persistence
    _cook_persistence = PersistencePluginManager.get_persistence_implementation(
        "cook_persistence_interface")


def add_cook(cook):
    _cook_persistence.save_cook(cook)


def remove_cook(cook):
    _cook_persistence.remove_cook(cook)
